    -- ResolverAD2S1200DecA7  - Resolver connected to AD2S1200 decoder with CmodA7 board
    -- Copyright (C) 2021 VCC
    -- release date: 31 Oct 2021
    -- author: VCC

    -- This file is part of ResolverAD2S1200DecA7 project.

    -- Permission is hereby granted, free of charge, to any person obtaining a copy
    -- of this software and associated documentation files (the "Software"),
    -- to deal in the Software without restriction, including without limitation
    -- the rights to use, copy, modify, merge, publish, distribute, sublicense,
    -- and/or sell copies of the Software, and to permit persons to whom the
    -- Software is furnished to do so, subject to the following conditions:

    -- The above copyright notice and this permission notice shall be included
    -- in all copies or substantial portions of the Software.

    -- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
    -- EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    -- MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
    -- IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
    -- DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
    -- TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
    -- OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


----------------------------------------------------------------------------------
-- Company: 
-- Engineer: VCC
-- 
-- Create Date:    20:21:40 10/17/2021 
-- Design Name: 
-- Module Name:    SlaveSPI_16bit_16bit - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity SlaveSPI_16bit_16bit is
    Port (Clk: in  STD_LOGIC;
          SO: out STD_LOGIC;
          SCK: in STD_LOGIC;
          SI: in STD_LOGIC;
          SS: in STD_LOGIC;
          SPIen: in STD_LOGIC;
           
          ParallelIn: in STD_LOGIC_VECTOR(15 downto 0);                      -- From FPGA to PIC
          ParallelOut: out STD_LOGIC_VECTOR(15 downto 0) := (others => '0'); -- From PIC to FPGA
          LoadData: out STD_LOGIC -- provides a 0 to 1 transition when to load the ParallelIn signal with data and use the data from ParallelOut signal
    );
end SlaveSPI_16bit_16bit;

architecture Behavioral of SlaveSPI_16bit_16bit is

  signal ShiftReg: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
  signal OutputBuffer: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
  signal OldSCK: STD_LOGIC := '0';
  signal OldSS: STD_LOGIC := '1';
  
  signal LoadDataTemp: STD_LOGIC := '0';
  signal OldLoadData: STD_LOGIC := '0';
  signal OldLoadData2: STD_LOGIC := '0';
  
  signal SyncSCK: STD_LOGIC;
  signal SyncSI: STD_LOGIC;
  signal SyncSS: STD_LOGIC;
  
  signal SyncSCK_SReg: STD_LOGIC_VECTOR(2 downto 0); -- 3-bit shift register, to cancel metastability
  signal SyncSI_SReg: STD_LOGIC_VECTOR(2 downto 0); -- 3-bit shift register, to cancel metastability
  signal SyncSS_SReg: STD_LOGIC_VECTOR(2 downto 0); -- 3-bit shift register, to cancel metastability
  
begin

  process (Clk)
  begin
    if Clk'event and Clk = '1' then
      SyncSCK_SReg <= SyncSCK_SReg(1 downto 0) & SCK;
      SyncSI_SReg <= SyncSI_SReg(1 downto 0) & SI;
      SyncSS_SReg <= SyncSS_SReg(1 downto 0) & SS;
    end if;
  end process;
  
  
  SyncSCK <= SyncSCK_SReg(2);
  SyncSI <= SyncSI_SReg(2);
  SyncSS <= SyncSS_SReg(2);


  process (Clk)
  begin
    if Clk'event and Clk = '1' then
      OldSCK <= SyncSCK;
      OldSS <= SyncSS;
      
      OldLoadData <= LoadDataTemp; --when OldLoadData is '1', ParallelIn is externally set
      OldLoadData2 <= OldLoadData;
      
      if OldLoadData2 = '1' then
        ShiftReg <= ParallelIn;
      elsif OldSCK = '0' and SyncSCK = '1' then
        if SPIen = '1' then
          ShiftReg <= ShiftReg(14 downto 0) & SyncSI;
        end if;
      end if;

      if OldSCK = '1' and SyncSCK = '0' then
        if SPIen = '1' then
          OutputBuffer <= ShiftReg(15 downto 0);  --LSBs
        end if;  
      end if;
      
    end if;
  end process;
  
  
  SO <= ShiftReg(15);
  LoadData <= LoadDataTemp;
  
  
  process (Clk)
  begin
    if Clk'event and Clk = '1' then
      if OldSS = '1' and SyncSS = '0' then --SS is asserting
        LoadDataTemp <= '1';
      else 
        LoadDataTemp <= '0';
      end if;
      
      if OldSS = '0' and SyncSS = '1' then  --SS is realeasing
        ParallelOut <= OutputBuffer;
      end if;
    end if;
  end process;

end Behavioral;

