    -- ResolverAD2S1200DecA7  - Resolver connected to AD2S1200 decoder with CmodA7 board
    -- Copyright (C) 2021 VCC
    -- release date: 31 Oct 2021
    -- author: VCC

    -- This file is part of ResolverAD2S1200DecA7 project.

    -- Permission is hereby granted, free of charge, to any person obtaining a copy
    -- of this software and associated documentation files (the "Software"),
    -- to deal in the Software without restriction, including without limitation
    -- the rights to use, copy, modify, merge, publish, distribute, sublicense,
    -- and/or sell copies of the Software, and to permit persons to whom the
    -- Software is furnished to do so, subject to the following conditions:

    -- The above copyright notice and this permission notice shall be included
    -- in all copies or substantial portions of the Software.

    -- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
    -- EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    -- MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
    -- IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
    -- DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
    -- TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
    -- OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

----------------------------------------------------------------------------------
-- Company: 
-- Engineer: VCC
-- 
-- Create Date:    10:46:20 10/17/2021 
-- Design Name: 
-- Module Name:    DecPosWithSlaveSPI - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity DecPosWithSlaveSPI is
  Port (Osc12MHz: in STD_LOGIC; --GCLK
    
        --AD2S1200 interface
        AD2S_LOT: in STD_LOGIC;
        AD2S_DOS: in STD_LOGIC;
        
        AD2S_RESET: out STD_LOGIC;
        AD2S_RD: out STD_LOGIC;
        AD2S_CS: out STD_LOGIC;
        AD2S_SAMPLE: out STD_LOGIC;
        AD2S_RDVEL: out STD_LOGIC;
        
        AD2S_SOE: out STD_LOGIC;
        AD2S_Data: in STD_LOGIC_VECTOR(11 downto 0);
        
        AD2S_FS1: out STD_LOGIC;
        AD2S_FS2: out STD_LOGIC;
        
        --PIC interface (FPGA is SPI slave)
        FPGA_SS: in STD_LOGIC;
        SCK: in STD_LOGIC;
        MOSI: in STD_LOGIC;
        MISO: out STD_LOGIC;
        
        --Parallel interface
        ParallelClk: in STD_LOGIC;                          --Clock signal from device reading from FPGA, using the parallel port. This clock is sampled by Clk100MHz, so it must be less than 50MHz.
        RDVel: in STD_LOGIC;                                --Data selector.  '0' for velocity, '1' for position
        ParallelData: out STD_LOGIC_VECTOR(11 downto 0);    --Can be position or velocity, depending on RDVel.
    
        --MISC pins
        LED_R: out STD_LOGIC; -- LOS
        LED_G: out STD_LOGIC; -- AD2S Reset command
        LED_B: out STD_LOGIC; -- Allow_AD2S_Reset_OnLOS from BTN_0
        LED_1: out STD_LOGIC;
        LED_2: out STD_LOGIC;
        
        BTN_0: in STD_LOGIC; -- sets Allow_AD2S_Reset_OnLOS, to cause the AD2S "reset FSM" to cycle
        BTN_1: in STD_LOGIC --sets the SPI data read by PIC to "1010111100001100"
    );
end DecPosWithSlaveSPI;

architecture Behavioral of DecPosWithSlaveSPI is

  component OscA7 is
    port (CLK_IN1: in std_logic;
          Clk100MHz: out std_logic;
          Clk1MHz: out std_logic;
          LOCKED: out std_logic
    );
  end component;
  
  component AD2SState is
    Port (Clk1MHz: in STD_LOGIC;                --Clock source for this component. Must be 1MHz, as timings are derived from it.
          Reset: in STD_LOGIC;                  --Reset signal for this component, active low. Can be tied to main "Locked" signal of the design.
          LOT: in STD_LOGIC;                    --Loss of track (output from AD2S1200), active low
          DOS: in STD_LOGIC;                    --Degradation of signal (output from AD2S1200), active low
          FilteredLOT: out STD_LOGIC;           --Persisted state of LOT, active high as a status indicator
          FilteredDOS: out STD_LOGIC;           --Persisted state of DOS, active high as a status indicator
          LOS: out STD_LOGIC;                   --Loss of signal (combination of LOT and DOS, when both are active), active high as a status indicator
          AD2S_Reset: out STD_LOGIC;            --AD2S1200 Reset signal, active 0
          Allow_AD2S_Reset_OnLOS: in STD_LOGIC; --Allows the state machine to reset the AD2S chip, active 1
          AD2S_Reset_FromUI: in STD_LOGIC;      --Manual reset from UI, active low
          ValidAfterReset: out STD_LOGIC        --Transitions to 1, when the next SAMPLE signal is allowed to go to 0  (see AD2S1200 startup sequence)
    );
  end component;
  
  component SlaveSPI_16bit_16bit is
    Port (Clk: in  STD_LOGIC;
          SO: out STD_LOGIC;
          SCK: in STD_LOGIC;
          SI: in STD_LOGIC;
          SS: in STD_LOGIC;
          SPIen: in STD_LOGIC;
           
          ParallelIn: in STD_LOGIC_VECTOR(15 downto 0);                      -- From FPGA to PIC
          ParallelOut: out STD_LOGIC_VECTOR(15 downto 0) := (others => '0'); -- From PIC to FPGA
          LoadData: out STD_LOGIC -- provides a 0 to 1 transition when to load the ParallelIn signal with data and use the data from ParallelOut signal
    );
  end component;
  
  component AD2S1200Interface is
    Port (
      Clk100MHz: in  STD_LOGIC;
      WantsToReadVelocity: in STD_LOGIC; -- when set to '1', the AD2S is asked about velocity
      
      --debugging signals, to obtain the right timing  (values controlled by UI)
      --uncomment the following "timing" signals if using the AD2S interface which supports them
      Timing_SampleOn0: in STD_LOGIC_VECTOR(7 downto 0); --initial value: 31
      Timing_RDCS0_ForPos: in STD_LOGIC_VECTOR(7 downto 0); --initial value: 81
      Timing_ReadPos: in STD_LOGIC_VECTOR(7 downto 0); --initial value: 88
      Timing_RDVELOutOn0_ForVel: in STD_LOGIC_VECTOR(7 downto 0); --initial value: 92
      Timing_RDCS0_ForVel: in STD_LOGIC_VECTOR(7 downto 0); --initial value: 95
      Timing_ReadVel: in STD_LOGIC_VECTOR(7 downto 0); --initial value: 102
      Timing_EndCycle: in STD_LOGIC_VECTOR(7 downto 0); --initial value: 107
      
      AD2SData: in STD_LOGIC_VECTOR(11 downto 0);
      SAMPLE: out STD_LOGIC;
      RD: out STD_LOGIC;
      CS: out STD_LOGIC;
      RDVELOut: out STD_LOGIC; --Read velocity/position  - '0' = velocity, '1' = position
           
      Position: out STD_LOGIC_VECTOR(11 downto 0);
      Velocity: out STD_LOGIC_VECTOR(11 downto 0)
    );
  end component;
  
  component ParallelInterface is
    Port (Clk100MHz: in STD_LOGIC;                            --Clock source for this component. Must be 100MHz
          
          --AD2S interface
          ResolverPosition: in STD_LOGIC_VECTOR(11 downto 0); --position from AD2S interface
          ResolverVelocity: in STD_LOGIC_VECTOR(11 downto 0); --velocity from AD2S interface
          
          --Misc
          ReadingVelocityRequest: out STD_LOGIC;              --becomes '0' when the velocity is not requested, so the AD2S interface won't even request velocity. This way, the position might be read more often. 
          
          --Parallel interface
          ParallelClk: in STD_LOGIC;                          --Clock signal from device reading from FPGA, using the parallel port. This clock is sampled by Clk100MHz, so it must be less than 50MHz.
          RDVel: in STD_LOGIC;                                --Data selector.  '0' for velocity, '1' for position
          ParallelData: out STD_LOGIC_VECTOR(11 downto 0)     --Can be position or velocity, depending on RDVel.
    );
  end component;
  
  
  signal Clk100MHz: STD_LOGIC;
  signal Clk1MHz: STD_LOGIC;
  signal OscLocked: STD_LOGIC;
  signal ValidAfterReset: STD_LOGIC;
    
  signal FPGAToPIC_Data: STD_LOGIC_VECTOR(15 downto 0); 
  signal PICToFPGA_Cmd: STD_LOGIC_VECTOR(15 downto 0);
  
  signal LoadData: STD_LOGIC := '0';
  
  signal ResolverPosition: STD_LOGIC_VECTOR(11 downto 0) := "101010101010"; --init with some value, useful for debugging
  signal ResolverVelocity: STD_LOGIC_VECTOR(11 downto 0) := "111110101111"; --init with some value, useful for debugging
  
  signal WantsToReadVelocity: STD_LOGIC := '1';
  signal ReadingVelocityRequestFromUI: STD_LOGIC := '1';
  signal ReadingVelocityRequestFromParallel: STD_LOGIC := '1';
  signal VelocityRequestCounter: STD_LOGIC_VECTOR(1 downto 0) := "00";
  
  signal SampleTemp: STD_LOGIC := '1';
  signal FS1_Temp: STD_LOGIC := '1';
  signal FS2_Temp: STD_LOGIC := '1';
  
  signal Allow_AD2S_Reset_OnLOS: STD_LOGIC := '1';
  signal LOS: STD_LOGIC;
  signal AD2S_Reset_Temp: STD_LOGIC;
  signal AD2S_Reset_FromUI: STD_LOGIC := '0';  -- active high

  signal FilteredLOT: STD_LOGIC := '0';
  signal FilteredDOS: STD_LOGIC := '0';
  
  
  signal Timing_SampleOn0: STD_LOGIC_VECTOR(7 downto 0) := CONV_STD_LOGIC_VECTOR(31, 8); --initial value: 31
  signal Timing_RDCS0_ForPos: STD_LOGIC_VECTOR(7 downto 0) := CONV_STD_LOGIC_VECTOR(81, 8); --initial value: 81
  signal Timing_ReadPos: STD_LOGIC_VECTOR(7 downto 0) := CONV_STD_LOGIC_VECTOR(88, 8); --initial value: 88
  signal Timing_RDVELOutOn0_ForVel: STD_LOGIC_VECTOR(7 downto 0) := CONV_STD_LOGIC_VECTOR(92, 8); --initial value: 92
  signal Timing_RDCS0_ForVel: STD_LOGIC_VECTOR(7 downto 0) := CONV_STD_LOGIC_VECTOR(95, 8); --initial value: 95
  signal Timing_ReadVel: STD_LOGIC_VECTOR(7 downto 0) := CONV_STD_LOGIC_VECTOR(102, 8); --initial value: 102
  signal Timing_EndCycle: STD_LOGIC_VECTOR(7 downto 0) := CONV_STD_LOGIC_VECTOR(107, 8); --initial value: 107
  
begin

  OscA7_Instance: OscA7 port map (
    CLK_IN1 => Osc12MHz,
    Clk100MHz => Clk100MHz,
    Clk1MHz => Clk1MHz,
    LOCKED => OscLocked
  );
  
  AD2S_SOE <= '1';  -- 1 - parallel interface
  
  AD2SState_Instance: AD2SState port map (
    Clk1MHz => Clk1MHz,
    Reset => OscLocked,
    LOT => AD2S_LOT,
    DOS => AD2S_DOS,
    FilteredLOT => FilteredLOT,
    FilteredDOS => FilteredDOS,
    LOS => LOS,
    AD2S_Reset => AD2S_Reset_Temp,
    Allow_AD2S_Reset_OnLOS => Allow_AD2S_Reset_OnLOS,
    AD2S_Reset_FromUI => AD2S_Reset_FromUI,
    ValidAfterReset => ValidAfterReset
  );
  
  
  LED_B <= not BTN_0;
  LED_R <= not LOS;
  LED_G <= '1'; --not Allow_AD2S_Reset_OnLOS;  --for debugging only
  AD2S_RESET <= AD2S_Reset_Temp; --active 0
  LED_1 <= FilteredLOT;
  LED_2 <= FilteredDOS;
  
  SlaveSPI_16bit_16bit_Instance: SlaveSPI_16bit_16bit port map (
    Clk => Clk100MHz,
    SO => MISO,
    SCK => SCK,
    SI => MOSI,
    SS => FPGA_SS,
    SPIen => OscLocked,
    ParallelIn => FPGAToPIC_Data,
    ParallelOut => PICToFPGA_Cmd,
    LoadData => LoadData
  );
  
  AD2S_SAMPLE <= (not ValidAfterReset) or SampleTemp; --------------------------------------------------------------------- add the reading logic
  
  AD2S1200Interface_Instance: AD2S1200Interface port map (
    Clk100MHz => Clk100MHz,
    WantsToReadVelocity => WantsToReadVelocity,
    
      Timing_SampleOn0 => Timing_SampleOn0,
      Timing_RDCS0_ForPos => Timing_RDCS0_ForPos,
      Timing_ReadPos => Timing_ReadPos,
      Timing_RDVELOutOn0_ForVel => Timing_RDVELOutOn0_ForVel,
      Timing_RDCS0_ForVel => Timing_RDCS0_ForVel,
      Timing_ReadVel => Timing_ReadVel,
      Timing_EndCycle => Timing_EndCycle,
    
    AD2SData => AD2S_Data,
    SAMPLE => SampleTemp,
    RD => AD2S_RD,
    CS => AD2S_CS,
    RDVELOut => AD2S_RDVEL,
    Position => ResolverPosition,
    Velocity => ResolverVelocity
  );
  
  
  AD2S_FS1 <= FS1_Temp;
  AD2S_FS2 <= FS2_Temp;
  WantsToReadVelocity <= ReadingVelocityRequestFromParallel or ReadingVelocityRequestFromUI;
  
  
  ParallelInterface_Instance: ParallelInterface port map (
    Clk100MHz => Clk100MHz,
    ResolverPosition => ResolverPosition,
    ResolverVelocity => ResolverVelocity,
    ReadingVelocityRequest => ReadingVelocityRequestFromParallel,
    ParallelClk => ParallelClk,
    RDVel => RDVel,
    ParallelData => ParallelData
  );
  
  process (Clk100MHz)
  begin
    if Clk100MHz'event and Clk100MHz = '1' then
      if LoadData = '1' then
      
        if BTN_1 = '1' then
          FPGAToPIC_Data <= "1010111100001100"; --use this to debug SPI
        else
          case PICToFPGA_Cmd(3 downto 0) is
            when "1010" => --"read position and status" command
              FPGAToPIC_Data <= '0' & LOS & FilteredDOS & FilteredLOT & ResolverPosition;
              --FPGAToPIC_Data <= "0000000000001001"; --use this to debug SPI
              
              VelocityRequestCounter <= VelocityRequestCounter + 1;
          
              if VelocityRequestCounter = 2 then
                VelocityRequestCounter <= "00";
                ReadingVelocityRequestFromUI <= '0';
              end if;
          
            when "1011" => --"read velocity" command
              FPGAToPIC_Data <= "0000" & ResolverVelocity;
              --FPGAToPIC_Data <= "0000100000000010"; --use this to debug SPI
              
              VelocityRequestCounter <= "00";
              ReadingVelocityRequestFromUI <= '1';
            
            when "1100" => --"set AD2S/resolver excitation frequency" command
              FS1_Temp <= not PICToFPGA_Cmd(9);
              FS2_Temp <= not PICToFPGA_Cmd(8);
              FPGAToPIC_Data <= (others => '0');
              
            when "1101" => --AD2S reset from UI
              AD2S_Reset_FromUI <= PICToFPGA_Cmd(8);
              
            when "1110" => --AD2S timings
              if PICToFPGA_Cmd(7 downto 4) = 0 then
                Timing_SampleOn0 <= PICToFPGA_Cmd(15 downto 8);
              end if;
              
              if PICToFPGA_Cmd(7 downto 4) = 1 then
                Timing_RDCS0_ForPos <= PICToFPGA_Cmd(15 downto 8);
              end if;
              
              if PICToFPGA_Cmd(7 downto 4) = 2 then
                Timing_ReadPos <= PICToFPGA_Cmd(15 downto 8);
              end if;
              
              if PICToFPGA_Cmd(7 downto 4) = 3 then
                Timing_RDVELOutOn0_ForVel <= PICToFPGA_Cmd(15 downto 8);
              end if;
                
              if PICToFPGA_Cmd(7 downto 4) = 4 then
                Timing_RDCS0_ForVel <= PICToFPGA_Cmd(15 downto 8);
              end if;
                
              if PICToFPGA_Cmd(7 downto 4) = 5 then
                Timing_ReadVel <= PICToFPGA_Cmd(15 downto 8);
              end if;
                
              if PICToFPGA_Cmd(7 downto 4) = 6 then
                Timing_EndCycle <= PICToFPGA_Cmd(15 downto 8);
              end if;
                      
            when others =>
              FPGAToPIC_Data <= (others => '1');
          end case;
        end if;
        
      end if;
    end if;
  end process;

end Behavioral;

